@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2014-2016 Tim Ruehsen
+ * Copyright(c) 2014-2018 Tim Ruehsen
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -839,12 +839,14 @@ static int _psl_is_public_suffix(const psl_ctx_t *psl, const char *domain, int t
 	}
 
 	if (suffix.nlabels == 1) {
-		/* TLD, this is the prevailing '*' match.
-		 * We don't currently support exception TLDs (TLDs that are not a public suffix)
+		/* TLD, this is the prevailing '*' match. If type excludes the '*' rule, continue.
 		 */
-		return 1;
+		if (!(type & PSL_TYPE_NO_STAR_RULE))
+			return 1;
 	}
 
+	type &= ~PSL_TYPE_NO_STAR_RULE;
+
 	if (psl->utf8 || psl == &_builtin_psl)
 		need_conversion = 0;
 
@@ -1002,7 +1004,11 @@ int psl_is_public_suffix(const psl_ctx_t *psl, const char *domain)
  * [Mozilla Public Suffix List](https://publicsuffix.org).
  *
  * @type specifies the PSL section where to perform the lookup. Valid values are
- * %PSL_TYPE_PRIVATE, %PSL_TYPE_ICANN and %PSL_TYPE_ANY.
+ * %PSL_TYPE_PRIVATE, %PSL_TYPE_ICANN, %PSL_TYPE_NO_STAR_RULE, and %PSL_TYPE_ANY.
+ *
+ * %PSL_TYPE_NO_STAR_RULE switches of the 'prevailing star rule' (see
+ * [List](https://publicsuffix.org/list) under 'Algorithm' 2.).
+ * Applying the flag means that TLDs not explicitly listed in the PSL are *not* treated as public suffixes.
  *
  * International @domain names have to be either in UTF-8 (lowercase + NFKC) or in ASCII/ACE format (punycode).
  * Other encodings likely result in incorrect return values.
@@ -1048,6 +1054,20 @@ const char *psl_unregistrable_domain(const psl_ctx_t *psl, const char *domain)
 	if (!psl || !domain)
 		return NULL;
 
+	/*
+	 * In the main loop we introduce a O(N^2) behavior to avoid code duplication.
+	 * To avoid nasty CPU hogging, we limit the lookup to max. 8 domain labels to the right.
+	 */
+
+	int nlabels = 0;
+	const char *p;
+	for (p = domain + strlen(domain) - 1; p >= domain; p--) {
+		if (*p == '.' && ++nlabels > 8) {
+			domain = p + 1;
+			break;
+		}
+	}
+
 	/*
 	 *  We check from left to right to catch special PSL entries like 'forgot.his.name':
 	 *   'forgot.his.name' and 'name' are in the PSL while 'his.name' is not.
@@ -1090,6 +1110,19 @@ const char *psl_registrable_domain(const psl_ctx_t *psl, const char *domain)
 	if (!psl || !domain || *domain == '.')
 		return NULL;
 
+	/*
+	 * In the main loop we introduce a O(N^2) behavior to avoid code duplication.
+	 * To avoid nasty CPU hogging, we limit the lookup to max. 8 domain labels to the right.
+	 */
+
+	int nlabels = 0;
+	for (p = domain + strlen(domain) - 1; p >= domain; p--) {
+		if (*p == '.' && ++nlabels > 8) {
+			domain = p + 1;
+			break;
+		}
+	}
+
 	/*
 	 *  We check from left to right to catch special PSL entries like 'forgot.his.name':
 	 *   'forgot.his.name' and 'name' are in the PSL while 'his.name' is not.
@@ -1249,8 +1282,6 @@ psl_ctx_t *psl_load_fp(FILE *fp)
 			psl->nwildcards++;
 			psl->nsuffixes++;
 		} else {
-			if (!strchr(p, '.'))
-				continue; /* we do not need an explicit plain TLD rule, already covered by implicit '*' rule */
 			suffix.flags = _PSL_FLAG_PLAIN | type;
 			psl->nsuffixes++;
 		}
@@ -1866,7 +1897,7 @@ static int _insert_file(const char *fname, const char **psl_fname, time_t *psl_m
  * - location specified during built-time (filename from ./configure --with-psl-distfile)
  * - built-in PSL data (generated from ./configure --with-psl-file)
  * - location of built-in data (filename from ./configure --with-psl-file)
- * 
+ *
  * If none of the above is available, the function returns %NULL.
  *
  * To free the allocated resources, call psl_free().
